use std::{fmt::Display, str::FromStr};

use chrono::NaiveDate;
use r2d2_sqlite::rusqlite::{
    self,
    types::{FromSql, FromSqlError},
    Row, ToSql,
};
use serde::{Deserialize, Serialize};

use super::{DbError, DbPool};

pub const EVENT_TYPES: [&str; 1] = ["Watering"];

#[derive(Serialize, Deserialize)]
pub enum EventType {
    Watering,
}

impl ToString for EventType {
    fn to_string(&self) -> String {
        String::from(match self {
            Self::Watering => "watering",
        })
    }
}

#[derive(Debug)]
pub struct ParseEventTypeErr;

impl Display for ParseEventTypeErr {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "invalid name")
    }
}

impl std::error::Error for ParseEventTypeErr {}

impl FromStr for EventType {
    type Err = ParseEventTypeErr;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "watering" => Ok(Self::Watering),
            _ => Err(ParseEventTypeErr),
        }
    }
}

impl ToSql for EventType {
    fn to_sql(&self) -> rusqlite::Result<rusqlite::types::ToSqlOutput<'_>> {
        Ok(self.to_string().into())
    }
}

impl FromSql for EventType {
    fn column_result(value: rusqlite::types::ValueRef<'_>) -> rusqlite::types::FromSqlResult<Self> {
        value
            .as_str()?
            .parse()
            .map_err(|e| FromSqlError::Other(Box::new(e)))
    }
}

#[derive(Serialize)]
pub struct Event {
    id: i32,
    plant_id: i32,
    event_type: EventType,
    date: NaiveDate,
    description: String,
}

impl Event {
    pub fn from_row(row: &Row<'_>) -> Result<Self, rusqlite::Error> {
        Ok(Self {
            id: row.get("id")?,
            plant_id: row.get("plant_id")?,
            event_type: row.get("event_type")?,
            date: row.get("date")?,
            description: row.get("description")?,
        })
    }
}

#[derive(Deserialize)]
pub struct NewEvent {
    plant_id: i32,
    event_type: EventType,
    date: NaiveDate,
    description: String,
}

impl NewEvent {
    pub fn insert(self, pool: &DbPool) -> Result<Event, DbError> {
        let conn = pool.get()?;

        let mut stmt = conn.prepare(
            "insert into events (plant_id, event_type, date, description) values ($1, $2, $3, $4) returning *",
        )?;

        Ok(stmt.query_row(
            (
                &self.plant_id,
                &self.event_type,
                &self.date,
                &self.description,
            ),
            Event::from_row,
        )?)
    }
}
