use r2d2_sqlite::rusqlite::{self, Row};
use serde::{Deserialize, Serialize};

use super::{Comment, DbError, DbPool, Event};

#[derive(Serialize)]
pub struct Plant {
    id: i32,
    name: String,
    species: String,
    description: String,
}

#[derive(Deserialize)]
pub struct NewPlant {
    name: String,
    species: String,
    description: String,
}

impl Plant {
    pub fn from_row(row: &Row<'_>) -> Result<Self, rusqlite::Error> {
        Ok(Self {
            id: row.get(0)?,
            name: row.get(1)?,
            species: row.get(2)?,
            description: row.get(3)?,
        })
    }

    pub fn all(pool: &DbPool) -> Result<Vec<Self>, DbError> {
        let conn = pool.get()?;

        let mut stmt = conn.prepare("select * from plants")?;
        let plants: Result<Vec<_>, _> = stmt.query_map((), Self::from_row)?.collect();

        Ok(plants?)
    }

    pub fn by_id(pool: &DbPool, id: i32) -> Result<Option<Self>, DbError> {
        let conn = pool.get()?;

        let mut stmt = conn.prepare("select * from plants where id = $1")?;
        match stmt.query_row((id,), |row| Plant::from_row(row)) {
            Ok(plant) => Ok(Some(plant)),
            Err(rusqlite::Error::QueryReturnedNoRows) => Ok(None),
            Err(err) => Err(DbError::Db(err)),
        }
    }

    pub fn comments(&self, pool: &DbPool) -> Result<Vec<Comment>, DbError> {
        let conn = pool.get()?;
        let mut stmt = conn.prepare("select * from comments where plant_id = $1")?;

        let comments: Result<Vec<_>, _> = stmt.query_map((self.id,), Comment::from_row)?.collect();
        Ok(comments?)
    }

    pub fn events(&self, pool: &DbPool) -> Result<Vec<Event>, DbError> {
        let conn = pool.get()?;
        let mut stmt = conn.prepare("select * from events where plant_id = $1")?;

        let events: Result<Vec<_>, _> = stmt.query_map((self.id,), Event::from_row)?.collect();
        Ok(events?)
    }
}

impl NewPlant {
    pub fn insert(self, pool: &DbPool) -> Result<Plant, DbError> {
        let conn = pool.get()?;

        let mut stmt = conn.prepare(
            "insert into plants (name, species, description) values ($1, $2, $3) returning *",
        )?;

        Ok(stmt.query_row(
            (&self.name, &self.species, &self.description),
            Plant::from_row,
        )?)
    }
}
