use std::fmt::{self, Write};

use axum::{http::StatusCode, response::IntoResponse};

use crate::db;

#[derive(Debug)]
pub enum AppError {
    Db(db::DbError),
    Tera(tera::Error),
    NotFound,
}

impl fmt::Display for AppError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Db(_) => write!(f, "database error"),
            Self::Tera(_) => write!(f, "error rendering template"),
            Self::NotFound => write!(f, "not found"),
        }
    }
}

impl std::error::Error for AppError {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Db(err) => Some(err),
            Self::Tera(err) => Some(err),
            Self::NotFound => None,
        }
    }
}

pub trait ErrorExt: std::error::Error {
    /// Return the full chain of error messages
    fn stack(&self) -> String {
        let mut msg = format!("{}", self);
        let mut err = self.source();

        while let Some(src) = err {
            write!(msg, " - {}", src).unwrap();

            err = src.source();
        }

        msg
    }
}

impl<E: std::error::Error> ErrorExt for E {}

impl From<db::DbError> for AppError {
    fn from(value: db::DbError) -> Self {
        Self::Db(value)
    }
}

impl From<tera::Error> for AppError {
    fn from(value: tera::Error) -> Self {
        Self::Tera(value)
    }
}

impl IntoResponse for AppError {
    fn into_response(self) -> axum::response::Response {
        match self {
            Self::NotFound => StatusCode::NOT_FOUND.into_response(),
            _ => {
                tracing::error!("{}", self.stack());

                StatusCode::INTERNAL_SERVER_ERROR.into_response()
            }
        }
    }
}
